@@ -86,7 +86,7 @@ const DEFAULT_TIMEOUT: u64 = 120;
 /// Default Client Version to communicate with the server.
 pub const API_DEFAULT_VERSION: &ClientVersion = &ClientVersion {
     major_version: 1,
-    minor_version: 49,
+    minor_version: 52,
 };
 
 #[derive(Debug, Clone)]
@@ -245,95 +245,43 @@ impl From<&(AtomicUsize, AtomicUsize)> for ClientVersion {
     }
 }
 
-pub(crate) fn serialize_as_json<T, S>(t: &T, s: S) -> Result<S::Ok, S::Error>
-where
-    T: Serialize,
-    S: serde::Serializer,
-{
-    s.serialize_str(
-        &serde_json::to_string(t).map_err(|e| serde::ser::Error::custom(format!("{e}")))?,
-    )
-}
+pub type RequestModifier = Arc<dyn Fn(BollardRequest) -> BollardRequest + Send + Sync>;
 
-pub(crate) fn serialize_join_newlines<S>(t: &[&str], s: S) -> Result<S::Ok, S::Error>
-where
-    S: serde::Serializer,
-{
-    s.serialize_str(&t.join("\n"))
-}
-
-#[cfg(feature = "time")]
-pub fn deserialize_rfc3339<'de, D: serde::Deserializer<'de>>(
-    d: D,
-) -> Result<time::OffsetDateTime, D::Error> {
-    let s: String = serde::Deserialize::deserialize(d)?;
-    time::OffsetDateTime::parse(&s, &time::format_description::well_known::Rfc3339)
-        .map_err(|e| serde::de::Error::custom(format!("{:?}", e)))
-}
-
-#[cfg(feature = "time")]
-pub fn serialize_rfc3339<S: serde::Serializer>(
-    date: &time::OffsetDateTime,
-    s: S,
-) -> Result<S::Ok, S::Error> {
-    s.serialize_str(
-        &date
-            .format(&time::format_description::well_known::Rfc3339)
-            .map_err(|e| serde::ser::Error::custom(format!("{:?}", e)))?,
-    )
-}
-
-#[cfg(feature = "time")]
-pub(crate) fn serialize_as_timestamp<S>(
-    opt: &Option<crate::models::BollardDate>,
-    s: S,
-) -> Result<S::Ok, S::Error>
-where
-    S: serde::Serializer,
-{
-    match opt {
-        Some(t) => s.serialize_str(&format!(
-            "{}.{}",
-            t.unix_timestamp(),
-            t.unix_timestamp_nanos()
-        )),
-        None => s.serialize_str(""),
-    }
-}
-
-#[cfg(all(feature = "chrono", not(feature = "time")))]
-pub(crate) fn serialize_as_timestamp<S>(
-    opt: &Option<crate::models::BollardDate>,
-    s: S,
-) -> Result<S::Ok, S::Error>
-where
-    S: serde::Serializer,
-{
-    match opt {
-        Some(t) => s.serialize_str(&format!("{}.{}", t.timestamp(), t.timestamp_subsec_nanos())),
-        None => s.serialize_str(""),
-    }
-}
-
-#[derive(Debug)]
 /// ---
 ///
 /// # Docker
 ///
 /// The main interface for calling the Docker API. Construct a new Docker instance using one of the
 /// connect methods:
-///  - [`Docker::connect_with_http_defaults`](Docker::connect_with_http_defaults())
-///  - [`Docker::connect_with_named_pipe_defaults`](Docker::connect_with_named_pipe_defaults())
-///  - [`Docker::connect_with_ssl_defaults`](Docker::connect_with_ssl_defaults())
-///  - [`Docker::connect_with_unix_defaults`](Docker::connect_with_unix_defaults())
-///  - [`Docker::connect_with_local_defaults`](Docker::connect_with_local_defaults())
-///  - [`Docker::connect_with_ssh_defaults`](Docker::connect_with_ssh_defaults())
+///  - [`Docker::connect_with_http_defaults`] (requires `http` feature)
+///  - `Docker::connect_with_named_pipe_defaults` (requires `pipe` feature, Windows only)
+///  - `Docker::connect_with_ssl_defaults` (requires `ssl` feature)
+///  - [`Docker::connect_with_unix_defaults`] (requires `pipe` feature, Unix only)
+///  - [`Docker::connect_with_local_defaults`]
+///  - `Docker::connect_with_ssh_defaults` (requires `ssh` feature)
 pub struct Docker {
     pub(crate) transport: Arc<Transport>,
     pub(crate) client_type: ClientType,
     pub(crate) client_addr: String,
     pub(crate) client_timeout: u64,
     pub(crate) version: Arc<(AtomicUsize, AtomicUsize)>,
+    pub(crate) request_modifier: Option<RequestModifier>,
+}
+
+impl std::fmt::Debug for Docker {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Docker")
+            .field("transport", &self.transport)
+            .field("client_type", &self.client_type)
+            .field("client_addr", &self.client_addr)
+            .field("client_timeout", &self.client_timeout)
+            .field("version", &self.version)
+            .field(
+                "request_modifier",
+                &self.request_modifier.as_ref().map(|_| "<callback>"),
+            )
+            .finish()
+    }
 }
 
 impl Clone for Docker {
@@ -344,6 +292,7 @@ impl Clone for Docker {
             client_addr: self.client_addr.clone(),
             client_timeout: self.client_timeout,
             version: self.version.clone(),
+            request_modifier: self.request_modifier.clone(),
         }
     }
 }
@@ -380,19 +329,34 @@ impl DockerClientCertResolver {
     }
 
     fn certs(path: &Path) -> Result<Vec<CertificateDer<'static>>, Error> {
-        Ok(rustls_pemfile::certs(&mut Self::open_buffered(path)?)
-            .collect::<Result<Vec<CertificateDer<'static>>, io::Error>>()?)
+        use rustls_pki_types::pem::PemObject;
+        rustls_pki_types::CertificateDer::pem_reader_iter(&mut Self::open_buffered(path)?)
+            .filter_map(|res| {
+                if matches!(res, Err(rustls_pki_types::pem::Error::NoItemsFound)) {
+                    None
+                } else {
+                    Some(res)
+                }
+            })
+            .collect::<Result<Vec<CertificateDer<'static>>, rustls_pki_types::pem::Error>>()
+            .map_err(|_| CertPathError {
+                path: path.to_path_buf(),
+            })
     }
 
     fn keys(path: &Path) -> Result<Vec<PrivateKeyDer<'static>>, Error> {
+        use rustls_pki_types::pem::PemObject;
         let mut rdr = Self::open_buffered(path)?;
         let mut keys = vec![];
-        if let Some(key) = rustls_pemfile::private_key(&mut rdr).map_err(|_| CertPathError {
-            path: path.to_path_buf(),
-        })? {
-            keys.push(key);
+        match rustls_pki_types::PrivateKeyDer::from_pem_reader(&mut rdr) {
+            Ok(key) => keys.push(key),
+            Err(rustls_pki_types::pem::Error::NoItemsFound) => {}
+            Err(_e) => {
+                return Err(CertPathError {
+                    path: path.to_path_buf(),
+                })
+            }
         }
-
         Ok(keys)
     }
 
@@ -522,6 +486,7 @@ impl Docker {
         timeout: u64,
         client_version: &ClientVersion,
     ) -> Result<Docker, Error> {
+        use rustls_pki_types::pem::PemObject;
         // This ensures that using docker-machine-esque addresses work with Hyper.
         let client_addr = addr.replacen("tcp://", "", 1).replacen("https://", "", 1);
 
@@ -550,7 +515,11 @@ impl Docker {
         })?);
 
         root_store.add_parsable_certificates(
-            rustls_pemfile::certs(&mut ca_pem).collect::<Result<Vec<_>, _>>()?,
+            rustls_pki_types::CertificateDer::pem_reader_iter(&mut ca_pem)
+                .collect::<Result<Vec<_>, _>>()
+                .map_err(|_| CertParseError {
+                    path: ssl_ca.to_owned(),
+                })?,
         );
 
         let config = rustls::ClientConfig::builder()
@@ -580,6 +549,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -659,6 +629,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -737,6 +708,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -819,10 +791,7 @@ impl Docker {
     ///
     /// This is a simple wrapper over the OS specific handlers:
     ///  * Unix: [`Docker::connect_with_unix_defaults`]
-    ///  * Windows: [`Docker::connect_with_named_pipe_defaults`]
-    ///
-    /// [`Docker::connect_with_unix_defaults`]: Docker::connect_with_unix_defaults()
-    /// [`Docker::connect_with_named_pipe_defaults`]: Docker::connect_with_named_pipe_defaults()
+    ///  * Windows: `Docker::connect_with_named_pipe_defaults`
     pub fn connect_with_local_defaults() -> Result<Docker, Error> {
         #[cfg(unix)]
         return Docker::connect_with_unix_defaults();
@@ -834,10 +803,7 @@ impl Docker {
     ///
     /// This is a simple wrapper over the OS specific handlers:
     ///  * Unix: [`Docker::connect_with_unix`]
-    ///  * Windows: [`Docker::connect_with_named_pipe`]
-    ///
-    /// [`Docker::connect_with_unix`]: Docker::connect_with_unix()
-    /// [`Docker::connect_with_named_pipe`]: Docker::connect_with_named_pipe()
+    ///  * Windows: `Docker::connect_with_named_pipe`
     pub fn connect_with_local(
         addr: &str,
         timeout: u64,
@@ -888,7 +854,7 @@ impl Docker {
             h if h.starts_with("https://") => Docker::connect_with_ssl_defaults(),
             #[cfg(feature = "ssh")]
             h if h.starts_with("ssh://") => {
-                Docker::connect_with_ssh(&h, DEFAULT_TIMEOUT, API_DEFAULT_VERSION)
+                Docker::connect_with_ssh(&h, DEFAULT_TIMEOUT, API_DEFAULT_VERSION, None)
             }
             _ => Err(UnsupportedURISchemeError {
                 uri: host.to_string(),
@@ -978,6 +944,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -1055,6 +1022,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -1086,7 +1054,7 @@ impl Docker {
     /// ```
     pub fn connect_with_ssh_defaults() -> Result<Docker, Error> {
         let host = env::var("DOCKER_HOST").unwrap_or_else(|_| DEFAULT_SSH_ADDRESS.to_string());
-        Docker::connect_with_ssh(&host, DEFAULT_TIMEOUT, API_DEFAULT_VERSION)
+        Docker::connect_with_ssh(&host, DEFAULT_TIMEOUT, API_DEFAULT_VERSION, None)
     }
 
     /// Connect using SSH.
@@ -1105,7 +1073,7 @@ impl Docker {
     /// use futures_util::future::TryFutureExt;
     ///
     /// let connection = Docker::connect_with_ssh(
-    ///                    "ssh://user@my-custom-docker-server", 4, API_DEFAULT_VERSION)
+    ///                    "ssh://user@my-custom-docker-server", 4, API_DEFAULT_VERSION, None)
     ///                    .unwrap();
     /// connection.ping()
     ///   .map_ok(|_| Ok::<_, ()>(println!("Connected!")));
@@ -1114,10 +1082,14 @@ impl Docker {
         addr: &str,
         timeout: u64,
         client_version: &ClientVersion,
+        keypair_path: Option<String>,
     ) -> Result<Docker, Error> {
         let client_addr = addr.replacen("ssh://", "", 1);
 
-        let ssh_connector = crate::ssh::SshConnector;
+        let ssh_connector = match keypair_path {
+            Some(path) => crate::ssh::SshConnector::with_keypair(path.to_string()),
+            None => crate::ssh::SshConnector::new(),
+        };
 
         let client_builder = Client::builder(TokioExecutor::new());
 
@@ -1132,6 +1104,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -1188,6 +1161,7 @@ impl Docker {
                 AtomicUsize::new(client_version.major_version),
                 AtomicUsize::new(client_version.minor_version),
             )),
+            request_modifier: None,
         };
 
         Ok(docker)
@@ -1220,6 +1194,32 @@ impl Docker {
     pub fn set_timeout(&mut self, timeout: Duration) {
         self.client_timeout = timeout.as_secs();
     }
+
+    /// Set a request modifier callback that runs before each request.
+    ///
+    /// This callback can modify the request before it is sent to the Docker Engine API.
+    /// Useful for setting `User-Agent` headers or other request modifications.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use bollard::Docker;
+    /// use http::header::{HeaderValue, USER_AGENT};
+    ///
+    /// let docker = Docker::connect_with_socket_defaults()
+    ///     .unwrap()
+    ///     .with_request_modifier(|mut req| {
+    ///         req.headers_mut().insert(USER_AGENT, HeaderValue::from_static("my-app/1.0"));
+    ///         req
+    ///     });
+    /// ```
+    pub fn with_request_modifier<F>(mut self, modifier: F) -> Self
+    where
+        F: Fn(BollardRequest) -> BollardRequest + Send + Sync + 'static,
+    {
+        self.request_modifier = Some(Arc::new(modifier));
+        self
+    }
 }
 
 // The implementation block for Docker requests
@@ -1444,10 +1444,19 @@ impl Docker {
         )?;
         let request_uri: hyper::Uri = uri.try_into()?;
         debug!("{}", &request_uri);
-        Ok(builder
+
+        let request = builder
             .uri(request_uri)
             .header(CONTENT_TYPE, "application/json")
-            .body(payload?)?)
+            .body(payload?)?;
+
+        let request = if let Some(modifier) = &self.request_modifier {
+            modifier(request)
+        } else {
+            request
+        };
+
+        Ok(request)
     }
 
     pub(crate) fn build_request_with_registry_auth<O>(
