@@ -1,16 +1,20 @@
 #![type_length_limit = "2097152"]
-#![allow(deprecated)]
 
+use bollard::container::AttachContainerResults;
+#[cfg(feature = "test_checkpoint")]
 use bollard::container::{
-    AttachContainerOptions, AttachContainerResults, Config, CreateContainerOptions,
-    DownloadFromContainerOptions, InspectContainerOptions, KillContainerOptions, LogsOptions,
-    PruneContainersOptions, RemoveContainerOptions, RenameContainerOptions,
-    ResizeContainerTtyOptions, RestartContainerOptions, StatsOptions, TopOptions,
-    UpdateContainerOptions, UploadToContainerOptions, WaitContainerOptions,
+    CreateCheckpointOptions, DeleteCheckpointOptions, ListCheckpointsOptions,
 };
 use bollard::errors::Error;
-use bollard::image::{CreateImageOptions, PushImageOptions, TagImageOptions};
-use bollard::query_parameters::ListContainersOptionsBuilder;
+#[cfg(feature = "test_checkpoint")]
+use bollard::query_parameters::RemoveContainerOptionsBuilder;
+use bollard::query_parameters::{
+    AttachContainerOptionsBuilder, ContainerArchiveInfoOptionsBuilder, CreateImageOptionsBuilder,
+    DownloadFromContainerOptionsBuilder, KillContainerOptionsBuilder, ListContainersOptionsBuilder,
+    LogsOptionsBuilder, PushImageOptionsBuilder, RenameContainerOptionsBuilder,
+    ResizeContainerTTYOptionsBuilder, StatsOptionsBuilder, TagImageOptionsBuilder,
+    TopOptionsBuilder, UploadToContainerOptionsBuilder,
+};
 use bollard::{body_full, Docker};
 use bollard::{body_stream, models::*};
 
@@ -45,10 +49,7 @@ async fn list_containers_test(docker: Docker) -> Result<(), Error> {
         .any(|container| container.image.as_ref().unwrap() == &image));
 
     let _ = &docker
-        .remove_container(
-            "integration_test_list_containers",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_list_containers", None)
         .await?;
 
     Ok(())
@@ -63,10 +64,11 @@ async fn image_push_test(docker: Docker) -> Result<(), Error> {
 
     let _ = &docker
         .create_image(
-            Some(CreateImageOptions {
-                from_image: &image[..],
-                ..Default::default()
-            }),
+            Some(
+                CreateImageOptionsBuilder::default()
+                    .from_image(&image)
+                    .build(),
+            ),
             None,
             if cfg!(windows) {
                 None
@@ -80,17 +82,18 @@ async fn image_push_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .tag_image(
             &image,
-            Some(TagImageOptions {
-                repo: format!("{}my-hello-world", registry_http_addr()),
-                ..Default::default()
-            }),
+            Some(
+                TagImageOptionsBuilder::default()
+                    .repo(&format!("{}my-hello-world", registry_http_addr()))
+                    .build(),
+            ),
         )
         .await?;
 
     let _ = &docker
         .push_image(
             format!("{}my-hello-world", registry_http_addr()).as_ref(),
-            None::<PushImageOptions<String>>,
+            Some(PushImageOptionsBuilder::default().build()),
             if cfg!(windows) {
                 None
             } else {
@@ -103,10 +106,11 @@ async fn image_push_test(docker: Docker) -> Result<(), Error> {
     let new_image_url = format!("{}my-hello-world", registry_http_addr());
     let _ = &docker
         .create_image(
-            Some(CreateImageOptions {
-                from_image: &new_image_url[..],
-                ..Default::default()
-            }),
+            Some(
+                CreateImageOptionsBuilder::default()
+                    .from_image(&new_image_url)
+                    .build(),
+            ),
             None,
             if cfg!(windows) {
                 None
@@ -124,25 +128,16 @@ async fn container_restart_test(docker: Docker) -> Result<(), Error> {
     create_daemon(&docker, "integration_test_restart_container").await?;
 
     let result = &docker
-        .inspect_container(
-            "integration_test_restart_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_restart_container", None)
         .await?;
 
     let started_at = result.state.as_ref().unwrap().started_at.as_ref();
 
     let _ = &docker
-        .restart_container(
-            "integration_test_restart_container",
-            None::<RestartContainerOptions>,
-        )
+        .restart_container("integration_test_restart_container", None)
         .await?;
     let result = &docker
-        .inspect_container(
-            "integration_test_restart_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_restart_container", None)
         .await?;
 
     assert_ne!(
@@ -158,7 +153,7 @@ async fn top_processes_test(docker: Docker) -> Result<(), Error> {
     let top_options = if cfg!(windows) {
         None
     } else {
-        Some(TopOptions { ps_args: "aux" })
+        Some(TopOptionsBuilder::default().ps_args("aux").build())
     };
 
     create_daemon(&docker, "integration_test_top_processes").await?;
@@ -183,13 +178,14 @@ async fn logs_test(docker: Docker) -> Result<(), Error> {
     &docker
         .logs(
             "integration_test_logs",
-            Some(LogsOptions {
-                follow: true,
-                stdout: true,
-                stderr: false,
-                tail: "all".to_string(),
-                ..Default::default()
-            }),
+            Some(
+                LogsOptionsBuilder::default()
+                    .follow(true)
+                    .stdout(true)
+                    .stderr(false)
+                    .tail("all")
+                    .build(),
+            ),
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -197,13 +193,14 @@ async fn logs_test(docker: Docker) -> Result<(), Error> {
     let vec = &docker
         .logs(
             "integration_test_logs",
-            Some(LogsOptions {
-                follow: true,
-                stdout: true,
-                stderr: false,
-                tail: "all".to_string(),
-                ..Default::default()
-            }),
+            Some(
+                LogsOptionsBuilder::default()
+                    .follow(true)
+                    .stdout(true)
+                    .stderr(false)
+                    .tail("all")
+                    .build(),
+            ),
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -213,7 +210,7 @@ async fn logs_test(docker: Docker) -> Result<(), Error> {
     assert_eq!(format!("{value}"), "Hello from Docker!\n".to_string());
 
     let _ = &docker
-        .remove_container("integration_test_logs", None::<RemoveContainerOptions>)
+        .remove_container("integration_test_logs", None)
         .await?;
 
     Ok(())
@@ -233,10 +230,7 @@ async fn container_changes_test(docker: Docker) -> Result<(), Error> {
     };
 
     let _ = &docker
-        .remove_container(
-            "integration_test_container_changes",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_container_changes", None)
         .await?;
 
     Ok(())
@@ -248,10 +242,7 @@ async fn stats_test(docker: Docker) -> Result<(), Error> {
     let vec = &docker
         .stats(
             "integration_test_stats",
-            Some(StatsOptions {
-                stream: false,
-                ..Default::default()
-            }),
+            Some(StatsOptionsBuilder::default().stream(false).build()),
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -270,7 +261,11 @@ async fn stats_test(docker: Docker) -> Result<(), Error> {
 }
 
 async fn kill_container_test(docker: Docker) -> Result<(), Error> {
-    let kill_options = Some(KillContainerOptions { signal: "SIGKILL" });
+    let kill_options = Some(
+        KillContainerOptionsBuilder::default()
+            .signal("SIGKILL")
+            .build(),
+    );
 
     create_daemon(&docker, "integration_test_kill_container").await?;
 
@@ -278,10 +273,7 @@ async fn kill_container_test(docker: Docker) -> Result<(), Error> {
         .kill_container("integration_test_kill_container", kill_options)
         .await?;
     let _ = &docker
-        .remove_container(
-            "integration_test_kill_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_kill_container", None)
         .await?;
 
     Ok(())
@@ -294,12 +286,13 @@ async fn attach_container_test(docker: Docker) -> Result<(), Error> {
     let AttachContainerResults { output, mut input } = docker
         .attach_container(
             "integration_test_attach_container",
-            Some(AttachContainerOptions::<String> {
-                stream: Some(true),
-                stdout: Some(true),
-                stdin: Some(true),
-                ..Default::default()
-            }),
+            Some(
+                AttachContainerOptionsBuilder::default()
+                    .stream(true)
+                    .stdout(true)
+                    .stdin(true)
+                    .build(),
+            ),
         )
         .await?;
 
@@ -314,28 +307,19 @@ async fn attach_container_test(docker: Docker) -> Result<(), Error> {
         Ok(res) => res?,
         Err(_) => {
             docker
-                .kill_container(
-                    "integration_test_attach_container",
-                    None::<KillContainerOptions<String>>,
-                )
+                .kill_container("integration_test_attach_container", None)
                 .await?;
             vec![]
         }
     };
 
     let _ = &docker
-        .wait_container(
-            "integration_test_attach_container",
-            None::<WaitContainerOptions<String>>,
-        )
+        .wait_container("integration_test_attach_container", None)
         .try_collect::<Vec<_>>()
         .await?;
 
     let _ = &docker
-        .remove_container(
-            "integration_test_attach_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_attach_container", None)
         .await?;
 
     let input_found = log
@@ -353,40 +337,31 @@ async fn resize_container_test(docker: Docker) -> Result<(), Error> {
     docker
         .resize_container_tty(
             "integration_test_resize_container_tty",
-            ResizeContainerTtyOptions {
-                width: 50,
-                height: 50,
-            },
+            ResizeContainerTTYOptionsBuilder::default()
+                .w(50)
+                .h(50)
+                .build(),
         )
         .await?;
 
     let _ = &docker
-        .kill_container(
-            "integration_test_resize_container_tty",
-            None::<KillContainerOptions<String>>,
-        )
+        .kill_container("integration_test_resize_container_tty", None)
         .await?;
 
     let _ = &docker
-        .wait_container(
-            "integration_test_resize_container_tty",
-            None::<WaitContainerOptions<String>>,
-        )
+        .wait_container("integration_test_resize_container_tty", None)
         .try_collect::<Vec<_>>()
         .await;
 
     let _ = &docker
-        .remove_container(
-            "integration_test_resize_container_tty",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_resize_container_tty", None)
         .await?;
 
     Ok(())
 }
 
 async fn update_container_test(docker: Docker) -> Result<(), Error> {
-    let update_options = UpdateContainerOptions::<String> {
+    let update_options = ContainerUpdateBody {
         memory: Some(314572800),
         memory_swap: Some(314572800),
         restart_policy: Some(RestartPolicy {
@@ -401,10 +376,7 @@ async fn update_container_test(docker: Docker) -> Result<(), Error> {
         .update_container("integration_test_update_container", update_options)
         .await?;
     let result = &docker
-        .inspect_container(
-            "integration_test_update_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_update_container", None)
         .await?;
 
     assert_eq!(
@@ -413,25 +385,16 @@ async fn update_container_test(docker: Docker) -> Result<(), Error> {
     );
 
     let _ = &docker
-        .kill_container(
-            "integration_test_update_container",
-            None::<KillContainerOptions<String>>,
-        )
+        .kill_container("integration_test_update_container", None)
         .await?;
 
     let _ = &docker
-        .wait_container(
-            "integration_test_update_container",
-            None::<WaitContainerOptions<String>>,
-        )
+        .wait_container("integration_test_update_container", None)
         .try_collect::<Vec<_>>()
         .await;
 
     let result = &docker
-        .inspect_container(
-            "integration_test_update_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_update_container", None)
         .await?;
 
     assert_eq!(
@@ -440,10 +403,7 @@ async fn update_container_test(docker: Docker) -> Result<(), Error> {
     );
 
     let _ = &docker
-        .remove_container(
-            "integration_test_update_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_update_container", None)
         .await?;
 
     Ok(())
@@ -454,17 +414,14 @@ async fn rename_container_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .rename_container(
             "integration_test_rename_container",
-            RenameContainerOptions {
-                name: "integration_test_rename_container_renamed".to_string(),
-            },
+            RenameContainerOptionsBuilder::default()
+                .name("integration_test_rename_container_renamed")
+                .build(),
         )
         .await?;
 
     let _ = &docker
-        .remove_container(
-            "integration_test_rename_container_renamed",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_rename_container_renamed", None)
         .await?;
 
     Ok(())
@@ -478,10 +435,7 @@ async fn pause_container_test(docker: Docker) -> Result<(), Error> {
         .await?;
 
     let result = &docker
-        .inspect_container(
-            "integration_test_pause_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_pause_container", None)
         .await?;
 
     assert_eq!(
@@ -494,10 +448,7 @@ async fn pause_container_test(docker: Docker) -> Result<(), Error> {
         .await?;
 
     let result = &docker
-        .inspect_container(
-            "integration_test_pause_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_pause_container", None)
         .await?;
 
     assert_eq!(
@@ -511,9 +462,7 @@ async fn pause_container_test(docker: Docker) -> Result<(), Error> {
 }
 
 async fn prune_containers_test(docker: Docker) -> Result<(), Error> {
-    let _ = &docker
-        .prune_containers(None::<PruneContainersOptions<String>>)
-        .await?;
+    let _ = &docker.prune_containers(None).await?;
 
     let result = &docker
         .list_containers(Some(
@@ -566,10 +515,11 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
 
     let _ = &docker
         .create_image(
-            Some(CreateImageOptions {
-                from_image: &image[..],
-                ..Default::default()
-            }),
+            Some(
+                CreateImageOptionsBuilder::default()
+                    .from_image(&image)
+                    .build(),
+            ),
             None,
             if cfg!(windows) {
                 None
@@ -582,12 +532,13 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
 
     let _ = &docker
         .create_container(
-            Some(CreateContainerOptions {
-                name: "integration_test_archive_container",
-                platform: None,
-            }),
-            Config {
-                image: Some(&image[..]),
+            Some(
+                bollard::query_parameters::CreateContainerOptionsBuilder::default()
+                    .name("integration_test_archive_container")
+                    .build(),
+            ),
+            ContainerCreateBody {
+                image: Some(image.to_owned()),
                 ..Default::default()
             },
         )
@@ -603,14 +554,15 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
         let _ = &docker
             .upload_to_container(
                 "integration_test_archive_container",
-                Some(UploadToContainerOptions {
-                    path: if cfg!(windows) {
-                        "C:\\Windows\\Logs"
-                    } else {
-                        "/tmp"
-                    },
-                    ..Default::default()
-                }),
+                Some(
+                    UploadToContainerOptionsBuilder::default()
+                        .path(if cfg!(windows) {
+                            "C:\\Windows\\Logs"
+                        } else {
+                            "/tmp"
+                        })
+                        .build(),
+                ),
                 body_stream(payload),
             )
             .await?;
@@ -618,28 +570,47 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
         let _ = &docker
             .upload_to_container(
                 "integration_test_archive_container",
-                Some(UploadToContainerOptions {
-                    path: if cfg!(windows) {
-                        "C:\\Windows\\Logs"
-                    } else {
-                        "/tmp"
-                    },
-                    ..Default::default()
-                }),
+                Some(
+                    UploadToContainerOptionsBuilder::default()
+                        .path(if cfg!(windows) {
+                            "C:\\Windows\\Logs"
+                        } else {
+                            "/tmp"
+                        })
+                        .build(),
+                ),
                 body_full(payload.into()),
             )
             .await?;
     }
+    let res = docker
+        .get_container_archive_info(
+            "integration_test_archive_container",
+            Some(
+                ContainerArchiveInfoOptionsBuilder::default()
+                    .path(if cfg!(windows) {
+                        "C:\\Windows\\Logs\\"
+                    } else {
+                        "/tmp"
+                    })
+                    .build(),
+            ),
+        )
+        .await?;
+
+    assert_eq!(res.name, if cfg!(windows) { "Logs" } else { "tmp" });
 
     let res = docker.download_from_container(
         "integration_test_archive_container",
-        Some(DownloadFromContainerOptions {
-            path: if cfg!(windows) {
-                "C:\\Windows\\Logs\\"
-            } else {
-                "/tmp"
-            },
-        }),
+        Some(
+            DownloadFromContainerOptionsBuilder::default()
+                .path(if cfg!(windows) {
+                    "C:\\Windows\\Logs\\"
+                } else {
+                    "/tmp"
+                })
+                .build(),
+        ),
     );
 
     let bytes = concat_byte_stream(res).await?;
@@ -677,10 +648,7 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
     assert_eq!("Hello from Bollard!", files.first().unwrap());
 
     let _ = &docker
-        .remove_container(
-            "integration_test_archive_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_archive_container", None)
         .await?;
 
     Ok(())
@@ -689,20 +657,15 @@ async fn archive_container_test(docker: Docker, streaming_upload: bool) -> Resul
 async fn inspect_container_test(docker: Docker) -> Result<(), Error> {
     create_daemon(&docker, "integration_test_inspect_container").await?;
     let result = &docker
-        .inspect_container(
-            "integration_test_inspect_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_inspect_container", None)
         .await?;
 
     assert_eq!(None, result.host_config.as_ref().unwrap().cap_add);
 
-    let config: Config<String> = result.config.as_ref().unwrap().to_owned().into();
+    // Just verify we can read the config directly
+    let config = result.config.as_ref().unwrap();
 
-    assert_eq!(
-        config.image.as_ref().unwrap(),
-        result.config.as_ref().unwrap().image.as_ref().unwrap()
-    );
+    assert!(config.image.is_some());
 
     kill_container(&docker, "integration_test_inspect_container").await?;
 
@@ -746,10 +709,11 @@ async fn mount_volume_container_test(docker: Docker) -> Result<(), Error> {
     };
 
     let _ = &docker.create_image(
-        Some(CreateImageOptions {
-            from_image: &image[..],
-            ..Default::default()
-        }),
+        Some(
+            CreateImageOptionsBuilder::default()
+                .from_image(&image)
+                .build(),
+        ),
         None,
         if cfg!(windows) {
             None
@@ -760,23 +724,22 @@ async fn mount_volume_container_test(docker: Docker) -> Result<(), Error> {
 
     let _ = &docker
         .create_container(
-            Some(CreateContainerOptions {
-                name: "integration_test_mount_volume_container",
-                platform: None,
-            }),
-            Config {
-                image: Some(&image[..]),
+            Some(
+                bollard::query_parameters::CreateContainerOptionsBuilder::default()
+                    .name("integration_test_mount_volume_container")
+                    .build(),
+            ),
+            ContainerCreateBody {
+                image: Some(image.clone()),
                 host_config: Some(host_config),
+                exposed_ports: Some(vec!["8080".to_string()]),
                 ..Default::default()
             },
         )
         .await?;
 
     let result = &docker
-        .inspect_container(
-            "integration_test_mount_volume_container",
-            None::<InspectContainerOptions>,
-        )
+        .inspect_container("integration_test_mount_volume_container", None)
         .await?;
 
     println!("{:?}", result.host_config);
@@ -821,11 +784,19 @@ async fn mount_volume_container_test(docker: Docker) -> Result<(), Error> {
             .unwrap()
     );
 
+    println!("{:?}", result.config);
+
+    assert!(result
+        .config
+        .as_ref()
+        .unwrap()
+        .exposed_ports
+        .as_ref()
+        .unwrap()
+        .contains(&"8080".to_string()));
+
     let _ = &docker
-        .remove_container(
-            "integration_test_mount_volume_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_mount_volume_container", None)
         .await?;
 
     Ok(())
@@ -853,10 +824,7 @@ async fn export_container_test(docker: Docker) -> Result<(), Error> {
     .await;
 
     docker
-        .remove_container(
-            "integration_test_export_container",
-            None::<RemoveContainerOptions>,
-        )
+        .remove_container("integration_test_export_container", None)
         .await?;
 
     // assert that the file containg the exported archive actually exists
@@ -967,3 +935,103 @@ fn integration_test_resize_container_tty() {
 fn integration_test_export_container() {
     connect_to_docker_and_run!(export_container_test);
 }
+
+#[cfg(feature = "test_checkpoint")]
+async fn checkpoint_test(docker: Docker) -> Result<(), Error> {
+    let image = format!("{}alpine:latest", registry_http_addr());
+
+    let _ = &docker
+        .create_image(
+            Some(
+                CreateImageOptionsBuilder::default()
+                    .from_image(&image)
+                    .build(),
+            ),
+            None,
+            Some(integration_test_registry_credentials()),
+        )
+        .try_collect::<Vec<_>>()
+        .await?;
+
+    let _ = &docker
+        .create_container(
+            Some(
+                bollard::query_parameters::CreateContainerOptionsBuilder::default()
+                    .name("integration_test_checkpoint")
+                    .build(),
+            ),
+            ContainerCreateBody {
+                image: Some(image.clone()),
+                cmd: Some(vec!["sleep".to_string(), "300".to_string()]),
+                ..Default::default()
+            },
+        )
+        .await?;
+
+    docker
+        .start_container("integration_test_checkpoint", None)
+        .await?;
+
+    let checkpoints = docker
+        .list_checkpoints(
+            "integration_test_checkpoint",
+            None::<ListCheckpointsOptions>,
+        )
+        .await?;
+    assert!(checkpoints.is_empty());
+
+    let create_options = CreateCheckpointOptions {
+        checkpoint_id: String::from("test-checkpoint-1"),
+        checkpoint_dir: None,
+        exit: false,
+    };
+    docker
+        .create_checkpoint("integration_test_checkpoint", create_options)
+        .await?;
+
+    let checkpoints = docker
+        .list_checkpoints(
+            "integration_test_checkpoint",
+            None::<ListCheckpointsOptions>,
+        )
+        .await?;
+    assert_eq!(1, checkpoints.len());
+    assert_eq!("test-checkpoint-1", checkpoints[0].name);
+
+    docker
+        .delete_checkpoint(
+            "integration_test_checkpoint",
+            "test-checkpoint-1",
+            None::<DeleteCheckpointOptions>,
+        )
+        .await?;
+
+    let checkpoints = docker
+        .list_checkpoints(
+            "integration_test_checkpoint",
+            None::<ListCheckpointsOptions>,
+        )
+        .await?;
+    assert!(checkpoints.is_empty());
+
+    docker
+        .kill_container("integration_test_checkpoint", None)
+        .await
+        .ok();
+
+    docker
+        .remove_container(
+            "integration_test_checkpoint",
+            Some(RemoveContainerOptionsBuilder::default().force(true).build()),
+        )
+        .await?;
+
+    Ok(())
+}
+
+#[test]
+#[cfg(feature = "test_checkpoint")]
+#[cfg(not(windows))]
+fn integration_test_checkpoint() {
+    connect_to_docker_and_run!(checkpoint_test);
+}
