@@ -1,17 +1,22 @@
-#![allow(deprecated)]
 use bytes::BufMut;
 use futures_util::future::ready;
 use futures_util::stream::{StreamExt, TryStreamExt};
 use http_body_util::Full;
 use tokio::runtime::Runtime;
 
-use bollard::container::{
-    Config, CreateContainerOptions, RemoveContainerOptions, StartContainerOptions,
-    WaitContainerOptions,
-};
+use bollard::body_full;
 use bollard::errors::Error;
+use bollard::models::{ContainerConfig, ContainerCreateBody};
+use bollard::query_parameters::{
+    BuildImageOptionsBuilder, CommitContainerOptionsBuilder, CreateContainerOptions,
+    CreateImageOptionsBuilder, ImportImageOptionsBuilder, ListImagesOptionsBuilder,
+    PruneImagesOptionsBuilder, RemoveContainerOptions, RemoveImageOptions,
+    RemoveImageOptionsBuilder, SearchImagesOptionsBuilder, StartContainerOptions,
+    TagImageOptionsBuilder, WaitContainerOptions,
+};
+#[cfg(feature = "buildkit")]
+use bollard::query_parameters::{BuilderVersion, ImageBuildOutput, PruneBuildOptions};
 use bollard::Docker;
-use bollard::{body_full, image::*};
 
 use std::collections::HashMap;
 use std::default::Default;
@@ -31,11 +36,10 @@ async fn create_image_test(docker: Docker) -> Result<(), Error> {
 async fn create_image_wasm_test(docker: Docker) -> Result<(), Error> {
     let image = "empty-wasm:latest";
 
-    let options = CreateImageOptions {
-        from_src: "-", // from_src must be "-" when sending the archive in the request body
-        repo: image,
-        ..Default::default()
-    };
+    let options = CreateImageOptionsBuilder::default()
+        .from_src("-") // from_src must be "-" when sending the archive in the request body
+        .repo(image)
+        .build();
 
     let req_body = bytes::Bytes::from({
         let mut buffer = Vec::new();
@@ -71,10 +75,11 @@ async fn create_image_wasm_test(docker: Docker) -> Result<(), Error> {
 
 async fn search_images_test(docker: Docker) -> Result<(), Error> {
     let result = &docker
-        .search_images(SearchImagesOptions {
-            term: "hello-world",
-            ..Default::default()
-        })
+        .search_images(
+            SearchImagesOptionsBuilder::default()
+                .term("hello-world")
+                .build(),
+        )
         .await?;
 
     assert!(result
@@ -115,10 +120,7 @@ async fn list_images_test(docker: Docker) -> Result<(), Error> {
     create_image_hello_world(&docker).await?;
 
     let result = &docker
-        .list_images(Some(ListImagesOptions::<String> {
-            all: true,
-            ..Default::default()
-        }))
+        .list_images(Some(ListImagesOptionsBuilder::default().all(true).build()))
         .await?;
 
     assert!(result.iter().any(|api_image| {
@@ -154,7 +156,11 @@ async fn prune_images_test(docker: Docker) -> Result<(), Error> {
     let mut filters = HashMap::new();
     filters.insert("label", vec!["maintainer=some_maintainer"]);
     let _ = &docker
-        .prune_images(Some(PruneImagesOptions { filters }))
+        .prune_images(Some(
+            PruneImagesOptionsBuilder::default()
+                .filters(&filters)
+                .build(),
+        ))
         .await?;
 
     Ok(())
@@ -172,10 +178,7 @@ async fn remove_image_test(docker: Docker) -> Result<(), Error> {
     let result = &docker
         .remove_image(
             &image,
-            Some(RemoveImageOptions {
-                noprune: true,
-                ..Default::default()
-            }),
+            Some(RemoveImageOptionsBuilder::default().noprune(true).build()),
             if cfg!(windows) {
                 None
             } else {
@@ -200,10 +203,20 @@ async fn commit_container_test(docker: Docker) -> Result<(), Error> {
         format!("{}alpine", registry_http_addr())
     };
 
-    let cmd = if cfg!(windows) {
-        Some(vec!["cmd.exe", "/C", "copy", "nul", "bollard.txt"])
+    let cmd: Option<Vec<String>> = if cfg!(windows) {
+        Some(
+            ["cmd.exe", "/C", "copy", "nul", "bollard.txt"]
+                .iter()
+                .map(|s| s.to_string())
+                .collect(),
+        )
     } else {
-        Some(vec!["touch", "/bollard.txt"])
+        Some(
+            ["touch", "/bollard.txt"]
+                .iter()
+                .map(|s| s.to_string())
+                .collect(),
+        )
     };
 
     create_image_hello_world(&docker).await?;
@@ -211,12 +224,12 @@ async fn commit_container_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_commit_container",
-                platform: None,
+                name: Some("integration_test_commit_container".to_string()),
+                ..Default::default()
             }),
-            Config {
-                cmd,
-                image: Some(&image[..]),
+            ContainerCreateBody {
+                cmd: cmd.clone(),
+                image: Some(image.clone()),
                 ..Default::default()
             },
         )
@@ -225,28 +238,27 @@ async fn commit_container_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .start_container(
             "integration_test_commit_container",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let _ = &docker
         .wait_container(
             "integration_test_commit_container",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
 
     let _ = &docker
         .commit_container(
-            CommitContainerOptions {
-                container: "integration_test_commit_container",
-                repo: "integration_test_commit_container_next",
-                tag: "latest",
-                pause: true,
-                ..Default::default()
-            },
-            Config::<String> {
+            CommitContainerOptionsBuilder::default()
+                .container("integration_test_commit_container")
+                .repo("integration_test_commit_container_next")
+                .tag("latest")
+                .pause(true)
+                .build(),
+            ContainerConfig {
                 ..Default::default()
             },
         )
@@ -255,15 +267,25 @@ async fn commit_container_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_commit_container_next",
-                platform: None,
+                name: Some("integration_test_commit_container_next".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_commit_container_next"),
+            ContainerCreateBody {
+                image: Some("integration_test_commit_container_next".to_string()),
                 cmd: if cfg!(windows) {
-                    Some(vec!["cmd.exe", "/C", "dir", "bollard.txt"])
+                    Some(
+                        ["cmd.exe", "/C", "dir", "bollard.txt"]
+                            .iter()
+                            .map(|s| s.to_string())
+                            .collect(),
+                    )
                 } else {
-                    Some(vec!["ls", "/bollard.txt"])
+                    Some(
+                        ["ls", "/bollard.txt"]
+                            .iter()
+                            .map(|s| s.to_string())
+                            .collect(),
+                    )
                 },
                 ..Default::default()
             },
@@ -273,14 +295,14 @@ async fn commit_container_test(docker: Docker) -> Result<(), Error> {
     let _ = &docker
         .start_container(
             "integration_test_commit_container_next",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_commit_container_next",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -362,13 +384,12 @@ RUN touch bollard.txt
 
         let _ = &docker
             .build_image(
-                BuildImageOptions {
-                    dockerfile: "Dockerfile".to_string(),
-                    t: "integration_test_build_image".to_string(),
-                    pull: true,
-                    rm: true,
-                    ..Default::default()
-                },
+                BuildImageOptionsBuilder::default()
+                    .dockerfile("Dockerfile")
+                    .t("integration_test_build_image")
+                    .pull("true")
+                    .rm(true)
+                    .build(),
                 if cfg!(windows) { None } else { Some(creds) },
                 Some(bollard::body_stream(payload)),
             )
@@ -377,13 +398,12 @@ RUN touch bollard.txt
     } else {
         let _ = &docker
             .build_image(
-                BuildImageOptions {
-                    dockerfile: "Dockerfile".to_string(),
-                    t: "integration_test_build_image".to_string(),
-                    pull: true,
-                    rm: true,
-                    ..Default::default()
-                },
+                BuildImageOptionsBuilder::default()
+                    .dockerfile("Dockerfile")
+                    .t("integration_test_build_image")
+                    .pull("true")
+                    .rm(true)
+                    .build(),
                 if cfg!(windows) { None } else { Some(creds) },
                 Some(http_body_util::Either::Left(Full::new(compressed.into()))),
             )
@@ -394,15 +414,25 @@ RUN touch bollard.txt
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_image",
-                platform: None,
+                name: Some("integration_test_build_image".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_image"),
+            ContainerCreateBody {
+                image: Some("integration_test_build_image".to_string()),
                 cmd: if cfg!(windows) {
-                    Some(vec!["cmd.exe", "/C", "dir", "bollard.txt"])
+                    Some(
+                        ["cmd.exe", "/C", "dir", "bollard.txt"]
+                            .iter()
+                            .map(|s| s.to_string())
+                            .collect(),
+                    )
                 } else {
-                    Some(vec!["ls", "/bollard.txt"])
+                    Some(
+                        ["ls", "/bollard.txt"]
+                            .iter()
+                            .map(|s| s.to_string())
+                            .collect(),
+                    )
                 },
                 ..Default::default()
             },
@@ -412,15 +442,12 @@ RUN touch bollard.txt
     let _ = &docker
         .start_container(
             "integration_test_build_image",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
-        .wait_container(
-            "integration_test_build_image",
-            None::<WaitContainerOptions<String>>,
-        )
+        .wait_container("integration_test_build_image", None::<WaitContainerOptions>)
         .try_collect::<Vec<_>>()
         .await?;
 
@@ -493,16 +520,14 @@ ENTRYPOINT ls buildkit-bollard.txt
 
         &docker
             .build_image(
-                BuildImageOptions {
-                    dockerfile: "Dockerfile".to_string(),
-                    t: "integration_test_build_buildkit_image".to_string(),
-                    pull: true,
-                    version: BuilderVersion::BuilderBuildKit,
-                    rm: true,
-                    #[cfg(feature = "buildkit_providerless")]
-                    session: Some(String::from(id)),
-                    ..Default::default()
-                },
+                BuildImageOptionsBuilder::default()
+                    .dockerfile("Dockerfile")
+                    .t("integration_test_build_buildkit_image")
+                    .pull("true")
+                    .version(BuilderVersion::BuilderBuildKit)
+                    .rm(true)
+                    .session(id)
+                    .build(),
                 Some(creds_hsh),
                 Some(body_stream(payload)),
             )
@@ -511,16 +536,14 @@ ENTRYPOINT ls buildkit-bollard.txt
     } else {
         &docker
             .build_image(
-                BuildImageOptions {
-                    dockerfile: "Dockerfile".to_string(),
-                    t: "integration_test_build_buildkit_image".to_string(),
-                    pull: true,
-                    version: BuilderVersion::BuilderBuildKit,
-                    rm: true,
-                    #[cfg(feature = "buildkit_providerless")]
-                    session: Some(String::from(id)),
-                    ..Default::default()
-                },
+                BuildImageOptionsBuilder::default()
+                    .dockerfile("Dockerfile")
+                    .t("integration_test_build_buildkit_image")
+                    .pull("true")
+                    .version(BuilderVersion::BuilderBuildKit)
+                    .rm(true)
+                    .session(id)
+                    .build(),
                 Some(creds_hsh),
                 Some(http_body_util::Either::Left(Full::new(compressed.into()))),
             )
@@ -546,12 +569,17 @@ ENTRYPOINT ls buildkit-bollard.txt
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_buildkit_image",
-                platform: None,
+                name: Some("integration_test_build_buildkit_image".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_buildkit_image"),
-                cmd: Some(vec!["ls", "/buildkit-bollard.txt"]),
+            ContainerCreateBody {
+                image: Some("integration_test_build_buildkit_image".to_string()),
+                cmd: Some(
+                    ["ls", "/buildkit-bollard.txt"]
+                        .iter()
+                        .map(|s| s.to_string())
+                        .collect(),
+                ),
                 ..Default::default()
             },
         )
@@ -560,14 +588,14 @@ ENTRYPOINT ls buildkit-bollard.txt
     let _ = &docker
         .start_container(
             "integration_test_build_buildkit_image",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_build_buildkit_image",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -620,16 +648,13 @@ ENTRYPOINT ls buildkit-bollard.txt
 
     let build = &docker
         .build_image(
-            BuildImageOptions {
-                dockerfile: "Dockerfile".to_string(),
-                t: "integration_test_build_buildkit_image".to_string(),
-                pull: true,
-                version: BuilderVersion::BuilderBuildKit,
-                rm: true,
-                #[cfg(feature = "buildkit_providerless")]
-                session: None,
-                ..Default::default()
-            },
+            BuildImageOptionsBuilder::default()
+                .dockerfile("Dockerfile")
+                .t("integration_test_build_buildkit_image")
+                .pull("true")
+                .version(BuilderVersion::BuilderBuildKit)
+                .rm(true)
+                .build(),
             None,
             Some(http_body_util::Either::Left(Full::new(compressed.into()))),
         )
@@ -710,12 +735,12 @@ COPY --from=builder1 /token /",
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_buildkit_secret",
-                platform: None,
+                name: Some("integration_test_build_buildkit_secret".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_buildkit_secret"),
-                cmd: Some(vec!["cat", "/token"]),
+            ContainerCreateBody {
+                image: Some("integration_test_build_buildkit_secret".to_string()),
+                cmd: Some(["cat", "/token"].iter().map(|s| s.to_string()).collect()),
                 ..Default::default()
             },
         )
@@ -724,14 +749,14 @@ COPY --from=builder1 /token /",
     let _ = &docker
         .start_container(
             "integration_test_build_buildkit_secret",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_build_buildkit_secret",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -745,11 +770,11 @@ COPY --from=builder1 /token /",
     let vec = &docker
         .logs(
             "integration_test_build_buildkit_secret",
-            Some(bollard::container::LogsOptions {
+            Some(bollard::query_parameters::LogsOptions {
                 follow: true,
                 stdout: true,
                 stderr: false,
-                tail: "all",
+                tail: "all".to_string(),
                 ..Default::default()
             }),
         )
@@ -901,12 +926,17 @@ async fn build_buildkit_named_context_test(docker: Docker) -> Result<(), Error>
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_buildkit_named_context",
-                platform: None,
+                name: Some("integration_test_build_buildkit_named_context".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_buildkit_named_context"),
-                cmd: Some(vec!["cat", "/etc/alpine-release"]),
+            ContainerCreateBody {
+                image: Some("integration_test_build_buildkit_named_context".to_string()),
+                cmd: Some(
+                    ["cat", "/etc/alpine-release"]
+                        .iter()
+                        .map(|s| s.to_string())
+                        .collect(),
+                ),
                 ..Default::default()
             },
         )
@@ -915,14 +945,14 @@ async fn build_buildkit_named_context_test(docker: Docker) -> Result<(), Error>
     let _ = &docker
         .start_container(
             "integration_test_build_buildkit_named_context",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_build_buildkit_named_context",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -1019,12 +1049,12 @@ RUN --mount=type=ssh git clone ssh://git@{}:{}/srv/git/config.git /config
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_buildkit_ssh",
-                platform: None,
+                name: Some("integration_test_build_buildkit_ssh".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_buildkit_ssh"),
-                cmd: Some(vec!["ls", "/config"]),
+            ContainerCreateBody {
+                image: Some("integration_test_build_buildkit_ssh".to_string()),
+                cmd: Some(["ls", "/config"].iter().map(|s| s.to_string()).collect()),
                 ..Default::default()
             },
         )
@@ -1033,14 +1063,14 @@ RUN --mount=type=ssh git clone ssh://git@{}:{}/srv/git/config.git /config
     let _ = &docker
         .start_container(
             "integration_test_build_buildkit_ssh",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_build_buildkit_ssh",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -1144,12 +1174,17 @@ ENTRYPOINT ls buildkit-bollard.txt
     let _ = &docker
         .create_container(
             Some(CreateContainerOptions {
-                name: "integration_test_build_buildkit_image_inline_driver",
-                platform: None,
+                name: Some("integration_test_build_buildkit_image_inline_driver".to_string()),
+                ..Default::default()
             }),
-            Config {
-                image: Some("integration_test_build_buildkit_image_inline_driver"),
-                cmd: Some(vec!["ls", "/buildkit-bollard.txt"]),
+            ContainerCreateBody {
+                image: Some("integration_test_build_buildkit_image_inline_driver".to_string()),
+                cmd: Some(
+                    ["ls", "/buildkit-bollard.txt"]
+                        .iter()
+                        .map(|s| s.to_string())
+                        .collect(),
+                ),
                 ..Default::default()
             },
         )
@@ -1158,14 +1193,14 @@ ENTRYPOINT ls buildkit-bollard.txt
     let _ = &docker
         .start_container(
             "integration_test_build_buildkit_image_inline_driver",
-            None::<StartContainerOptions<String>>,
+            None::<StartContainerOptions>,
         )
         .await?;
 
     let vec = &docker
         .wait_container(
             "integration_test_build_buildkit_image_inline_driver",
-            None::<WaitContainerOptions<String>>,
+            None::<WaitContainerOptions>,
         )
         .try_collect::<Vec<_>>()
         .await?;
@@ -1280,20 +1315,17 @@ COPY --from=builder message-2.txt .
     let id = "build_buildkit_image_outputs_tar_test";
     let build = &docker
         .build_image(
-            BuildImageOptions {
-                dockerfile: "Dockerfile".to_string(),
-                t: "integration_test_build_buildkit_image".to_string(),
-                pull: true,
-                version: BuilderVersion::BuilderBuildKit,
-                rm: true,
-                #[cfg(feature = "buildkit_providerless")]
-                session: Some(String::from(id)),
-                #[cfg(feature = "buildkit_providerless")]
-                outputs: Some(ImageBuildOutput::Tar(
+            BuildImageOptionsBuilder::default()
+                .dockerfile("Dockerfile")
+                .t("integration_test_build_buildkit_image")
+                .pull("true")
+                .version(BuilderVersion::BuilderBuildKit)
+                .rm(true)
+                .session(id)
+                .outputs(ImageBuildOutput::Tar(
                     "/tmp/buildkit-outputs.tar".to_string(),
-                )),
-                ..Default::default()
-            },
+                ))
+                .build(),
             Some(creds_hsh),
             Some(http_body_util::Either::Left(Full::new(compressed.into()))),
         )
@@ -1389,18 +1421,15 @@ RUN touch empty.txt
     let id = "build_buildkit_image_outputs_local_test";
     let build = &docker
         .build_image(
-            BuildImageOptions {
-                dockerfile: "Dockerfile".to_string(),
-                t: "integration_test_build_buildkit_image".to_string(),
-                pull: true,
-                version: BuilderVersion::BuilderBuildKit,
-                rm: true,
-                #[cfg(feature = "buildkit_providerless")]
-                session: Some(String::from(id)),
-                #[cfg(feature = "buildkit_providerless")]
-                outputs: Some(ImageBuildOutput::Local("/tmp/buildkit-outputs".to_string())),
-                ..Default::default()
-            },
+            BuildImageOptionsBuilder::default()
+                .dockerfile("Dockerfile")
+                .t("integration_test_build_buildkit_image")
+                .pull("true")
+                .version(BuilderVersion::BuilderBuildKit)
+                .rm(true)
+                .session(id)
+                .outputs(ImageBuildOutput::Local("/tmp/buildkit-outputs".to_string()))
+                .build(),
             Some(creds_hsh),
             Some(http_body_util::Either::Left(Full::new(compressed.into()))),
         )
@@ -1476,16 +1505,14 @@ RUN echo bollard > bollard.txt
 
     let _ = &docker
         .build_image(
-            BuildImageOptions {
-                dockerfile: "Dockerfile".to_string(),
-                t: "integration_test_prune_build".to_string(),
-                pull: true,
-                version: BuilderVersion::BuilderBuildKit,
-                rm: true,
-                #[cfg(feature = "buildkit_providerless")]
-                session: Some(String::from(id)),
-                ..Default::default()
-            },
+            BuildImageOptionsBuilder::default()
+                .dockerfile("Dockerfile")
+                .t("integration_test_prune_build")
+                .pull("true")
+                .version(BuilderVersion::BuilderBuildKit)
+                .rm(true)
+                .session(id)
+                .build(),
             Some(creds_hsh),
             Some(http_body_util::Either::Left(Full::new(compressed.into()))),
         )
@@ -1502,30 +1529,24 @@ RUN echo bollard > bollard.txt
         )
         .await?;
 
-    let old_cache_size = &docker
+    let old_cache_size = docker
         .df(None::<DataUsageOptions>)
         .await?
-        .build_cache
-        .map(|data| data.iter().fold(0, |acc, e| acc + e.size.unwrap()))
-        .unwrap();
+        .build_cache_disk_usage
+        .and_then(|data| data.total_size)
+        .unwrap_or(0);
 
-    let prune_info = docker
-        .prune_build(None::<PruneBuildOptions<String>>)
-        .await?;
+    let prune_info = docker.prune_build(None::<PruneBuildOptions>).await?;
 
-    let new_cache_size = &docker
+    let new_cache_size = docker
         .df(None::<DataUsageOptions>)
         .await?
-        .build_cache
-        .map(|data| data.iter().fold(0, |acc, e| acc + e.size.unwrap()))
-        .unwrap();
+        .build_cache_disk_usage
+        .and_then(|data| data.total_size)
+        .unwrap_or(0);
 
-    assert!(old_cache_size - new_cache_size > 0);
-    assert_eq!(
-        prune_info.space_reclaimed,
-        Some(old_cache_size - new_cache_size)
-    );
-    assert!(prune_info.caches_deleted.is_some_and(|c| !c.is_empty()));
+    assert!(old_cache_size >= new_cache_size);
+    assert!(prune_info.caches_deleted.is_some());
 
     Ok(())
 }
@@ -1575,10 +1596,12 @@ async fn export_images_test(docker: Docker) -> Result<(), Error> {
     docker
         .tag_image(
             &image,
-            Some(TagImageOptions {
-                repo: repo.as_ref(),
-                tag: "mycopy",
-            }),
+            Some(
+                TagImageOptionsBuilder::default()
+                    .repo(repo.as_ref())
+                    .tag("mycopy")
+                    .build(),
+            ),
         )
         .await?;
 
@@ -1643,13 +1666,12 @@ RUN apt-get update && \
     let compressed = c.finish().unwrap();
 
     let mut stream = docker.build_image(
-        BuildImageOptions {
-            dockerfile: "Dockerfile".to_string(),
-            t: "issue_55".to_string(),
-            pull: true,
-            rm: true,
-            ..Default::default()
-        },
+        BuildImageOptionsBuilder::default()
+            .dockerfile("Dockerfile")
+            .t("issue_55")
+            .pull("true")
+            .rm(true)
+            .build(),
         None,
         Some(http_body_util::Either::Left(Full::new(compressed.into()))),
     );
@@ -1686,9 +1708,7 @@ async fn import_image_test(docker: Docker) -> Result<(), Error> {
 
     docker
         .import_image(
-            ImportImageOptions {
-                ..Default::default()
-            },
+            ImportImageOptionsBuilder::default().build(),
             body_full(buf.freeze()),
             Some(creds),
         )
@@ -1718,10 +1738,8 @@ async fn import_image_test_stream(docker: Docker) -> Result<(), Error> {
 
     docker
         .import_image_stream(
-            ImportImageOptions {
-                ..Default::default()
-            },
-            res.map(|b| b.unwrap()),
+            ImportImageOptionsBuilder::default().build(),
+            res,
             Some(creds),
         )
         .try_collect::<Vec<_>>()
