@@ -25,6 +25,25 @@ const ETA2_RANDOMNESS_SIZE: usize = ETA2 * 64;
 
 const IMPLICIT_REJECTION_HASH_INPUT_SIZE: usize = SHARED_SECRET_SIZE + CPA_PKE_CIPHERTEXT_SIZE;
 
+/// The ML-KEM 1024 algorithms
+pub struct MlKem1024;
+
+#[cfg(not(any(hax, eurydice)))]
+crate::impl_kem_trait!(
+    MlKem1024,
+    MlKem1024PublicKey,
+    MlKem1024PrivateKey,
+    MlKem1024Ciphertext
+);
+
+// Provide the (packed) PQCP APIs
+#[cfg(feature = "pqcp")]
+crate::pqcp::pqcp_api!(
+    "use libcrux_ml_kem::mlkem1024::pqcp::*;",
+    MlKem1024,
+    " 1024 "
+);
+
 /// An ML-KEM 1024 Ciphertext
 pub type MlKem1024Ciphertext = MlKemCiphertext<CPA_PKE_CIPHERTEXT_SIZE>;
 /// An ML-KEM 1024 Private key
@@ -239,6 +258,16 @@ macro_rules! instantiate {
                 /// Am Unpacked ML-KEM 1024 Key pair
                 pub type MlKem1024KeyPairUnpacked = p::unpacked::MlKemKeyPairUnpacked<RANK>;
 
+                #[cfg(feature = "pqcp")]
+                crate::pqcp::pqcp_unpacked_api!(
+                    MlKem1024KeyPairUnpacked,
+                    MlKem1024PublicKeyUnpacked,
+                    MlKem1024PrivateKey,
+                    MlKem1024PublicKey,
+                    MlKem1024Ciphertext,
+                    " 1024 "
+                );
+
                 /// Create a new, empty unpacked key.
                 pub fn init_key_pair() -> MlKem1024KeyPairUnpacked {
                     MlKem1024KeyPairUnpacked::default()
@@ -578,6 +607,16 @@ pub mod rand {
 pub(crate) mod kyber {
     use super::*;
 
+    /// The Kyber 1024 algorithms
+    pub struct Kyber1024;
+
+    crate::impl_kem_trait!(
+        Kyber1024,
+        MlKem1024PublicKey,
+        MlKem1024PrivateKey,
+        MlKem1024Ciphertext
+    );
+
     /// Generate Kyber 1024 Key Pair
     ///
     /// Generate an ML-KEM key pair. The input is a byte array of size
