@@ -5,8 +5,7 @@ use opentelemetry::{
     trace::{SpanContext, Status, TraceContextExt},
     Context, Key, KeyValue, Value,
 };
-use std::{borrow::Cow, time::SystemTime};
-use thiserror::Error;
+use std::{borrow::Cow, fmt, time::SystemTime};
 
 /// Utility functions to allow tracing [`Span`]s to accept and return
 /// [OpenTelemetry] [`Context`]s.
@@ -226,16 +225,44 @@ pub trait OpenTelemetrySpanExt {
     );
 }
 
-#[derive(Error, Debug)]
+/// An error returned if [`OpenTelemetrySpanExt::set_parent`] could not set the parent.
+#[derive(Debug)]
 pub enum SetParentError {
-    #[error("OpenTelemetry layer not found")]
+    /// The layer could not be found and therefore the action could not be carried out. This can
+    /// happen with some advanced layers that do not handle downcasting well, for example
+    /// [`tracing_subscriber::reload::Layer`].
     LayerNotFound,
-    #[error("Span has already been started, cannot set parent")]
+
+    /// The span has been already started.
+    ///
+    /// Someone already called a context-starting method such as [`OpenTelemetrySpanExt::context`]
+    /// or the span has been entered and automatic context starting was not configured out.
     AlreadyStarted,
-    #[error("Span disabled")]
+
+    /// The span is filtered out by tracing filters.
+    ///
+    /// If the filtered out span had children, they will not be connected to the parent span either.
     SpanDisabled,
 }
 
+impl fmt::Display for SetParentError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            SetParentError::LayerNotFound => {
+                write!(f, "OpenTelemetry layer not found")
+            }
+            SetParentError::AlreadyStarted => {
+                write!(f, "Span has already been started, cannot set parent")
+            }
+            SetParentError::SpanDisabled => {
+                write!(f, "Span disabled")
+            }
+        }
+    }
+}
+
+impl std::error::Error for SetParentError {}
+
 impl OpenTelemetrySpanExt for tracing::Span {
     fn set_parent(&self, cx: Context) -> Result<(), SetParentError> {
         let mut cx = Some(cx);
@@ -363,8 +390,8 @@ impl OpenTelemetrySpanExt for tracing::Span {
                 return;
             };
             get_context.with_context(subscriber, id, move |data| match &mut data.state {
-                OtelDataState::Builder { builder, .. } => {
-                    builder.status = status.take().unwrap();
+                OtelDataState::Builder { status: s, .. } => {
+                    *s = status.take().unwrap();
                 }
                 OtelDataState::Context { current_cx } => {
                     let span = current_cx.span();
